%% Simulation for endowment economy
clc
%%%%%%%%%%%%% Parameters %%%%%%%%%%%%%%%%
T=200000;
cut=1000;  % Initial condition dependence

%% 1. Simulation

[S_index,]=markov(Prob,T+cut+1,1,1:length(Prob));

bpSIM=zeros(T+cut,1);
WfSIM=zeros(T+cut,1);
bindSIM=zeros(T+cut,1);
cSIM=zeros(T+cut,1);
pSIM=zeros(T+cut,1);

bpSP_SIM=zeros(T+cut,1);
bindSP_SIM=zeros(T+cut,1);
cSP_SIM=zeros(T+cut,1);
pSP_SIM=zeros(T+cut,1);

Tao_SIM=zeros(T+cut,1);
Tregion_SIM=zeros(T+cut,1);
ySIM=zeros(T+cut,1);
RSIM=zeros(T+cut,1);

bpSIM(1)=bmax+(bmin-bmax)/2;
bpSP_SIM(1)=bmax+(bmin-bmax)/2;

for i=2:T+cut
    bpSIM(i)=interp1(B,bp(:,S_index(i)),bpSIM(i-1),'linear');
    WfSIM(i)=interp1(B,Wf(:,S_index(i)),bpSIM(i-1),'linear');
    ySIM(i)=yt(1,S_index(i));  
    RSIM(i)=R(1,S_index(i));  
    cSIM(i)=bpSIM(i-1)+ySIM(i)-(1/(1+RSIM(i)))*bpSIM(i); 
    bpSP_SIM(i)=interp1(B,bpSP(:,S_index(i)),bpSP_SIM(i-1),'linear');
    cSP_SIM(i)=bpSP_SIM(i-1)+ySIM(i)-(1/(1+RSIM(i)))*bpSP_SIM(i); 
    Tao_SIM(i)=interp1(B,Tao(:,S_index(i)),bpSP_SIM(i-1),'linear');
end

pSIM=(1-omega)/omega*cSIM.^(1+ita); 
pSP_SIM=(1-omega)/omega*cSP_SIM.^(1+ita);

bplim_SIM=-(1+RSIM).*kappa.*(pSIM+ySIM);
bplimSP_SIM=-(1+RSIM).*kappa.*(pSP_SIM+ySIM);

%%%%% IMPORTANT TO USE TOLERANCE NOT <= %%%%%%%%%%%%
bindSIM((bpSIM-bplim_SIM)<=1e-5)=1;
bindSP_SIM((bpSP_SIM-bplimSP_SIM)<=1e-5)=1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bpSIM(bindSIM==1)=bplim_SIM(bindSIM==1);
bpSP_SIM(bindSP_SIM==1)=bplimSP_SIM(bindSP_SIM==1);

Tregion_SIM(Tao_SIM>1)=1;

CA_SIM=bpSIM(2:end)-bpSIM(1:end-1);  % >0 capital outflow
CASP_SIM=bpSP_SIM(2:end)-bpSP_SIM(1:end-1);

% Eliminate Initial Condition Dependency
S_index=S_index(cut+1:T+cut)';

bpSIM=bpSIM(cut+1:T+cut);
bindSIM=bindSIM(cut+1:T+cut);
bplim_SIM=bplim_SIM(cut+1:T+cut);
cSIM=cSIM(cut+1:T+cut);
pSIM=pSIM(cut+1:T+cut);
CA_SIM=CA_SIM(cut:T+cut-1);

bpSP_SIM=bpSP_SIM(cut+1:T+cut);
bindSP_SIM=bindSP_SIM(cut+1:T+cut);
bplimSP_SIM=bplimSP_SIM(cut+1:T+cut);
cSP_SIM=cSP_SIM(cut+1:T+cut);
pSP_SIM=pSP_SIM(cut+1:T+cut);
CASP_SIM=CASP_SIM(cut:T+cut-1);

WfSIM=WfSIM(cut+1:T+cut);
Tao_SIM=Tao_SIM(cut+1:T+cut);
Tregion_SIM=Tregion_SIM(cut+1:T+cut);
ySIM=ySIM(cut+1:T+cut);
RSIM=RSIM(cut+1:T+cut);

Y_SIM=pSIM+ySIM;   % Income
YSP_SIM=pSP_SIM+ySIM;

CA_SIM=CA_SIM./Y_SIM;  % Current Account
CASP_SIM=CASP_SIM./YSP_SIM;

CAchg_SIM=CA_SIM(2:end)-CA_SIM(1:end-1);
CAchg_SIM=[0;CAchg_SIM];

Lev_SIM=bpSIM./Y_SIM;  % Leverage
LevSP_SIM=bpSP_SIM./YSP_SIM;

RER_SIM=(omega^(1/(1+ita))+(1-omega)^(1/(1+ita)).*pSIM.^(ita/(1+ita))).^((-1-ita)/ita); % Real Exchange Rate
RERSP_SIM=(omega^(1/(1+ita))+(1-omega)^(1/(1+ita)).*pSP_SIM.^(ita/(1+ita))).^((-1-ita)/ita);
%% 2. Event Analysis
%%%%%%% Parameter %%%%%%%%%%%%%%%%%%%%
nbd=3; % periods before and after crisis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
clear SS
SS=zeros(T,1);
SS_SP=zeros(T,1);
SS(CA_SIM>(mean(CA_SIM)+2*std(CA_SIM)) & bindSIM>0)=1; % Crisis is defined as current account goes 2 sd away and collateral constraint binds in decentralized economy
SS_SP(CASP_SIM>(mean(CA_SIM)+2*std(CA_SIM)) & bindSP_SIM>0)=1;

fprintf(' \n ');
fprintf('The long-run probability of crisis in DE is: ');
fprintf('%5.4f \n ',  mean(SS));
fprintf('The long-run probability of crisis in SP is: ');
fprintf('%5.4f \n ',  mean(SS_SP));

fprintf('std of CA/Y DE is: ');
fprintf('%5.4f \n ',  std(CA_SIM));

fprintf('std of CA/Y SP is: ');
fprintf('%5.4f \n ',  std(CASP_SIM));

fprintf('mean debt/Y level in DE is: ');
fprintf('%5.4f \n ',  mean(Lev_SIM));

fprintf('mean debt/Y level in SP is: ');
fprintf('%5.4f \n ',  mean(LevSP_SIM));

fprintf('welfare gain is: ');
fprintf('%.6f \n ',  mean(WfSIM));

CtSIM=(omega.*cSIM.^(-ita)+(1-omega)).^(-1/ita);
CtSPSIM=(omega.*cSP_SIM.^(-ita)+(1-omega)).^(-1/ita);

Cdev=(CtSIM-mean(CtSIM))./mean(CtSIM);
CSPdev=(CtSPSIM-mean(CtSPSIM))./mean(CtSPSIM);

RERdev=(RER_SIM-mean(RER_SIM))./mean(RER_SIM);
RERSPdev=(RERSP_SIM-mean(RERSP_SIM))./mean(RERSP_SIM);
Ydev=(Y_SIM-mean(Y_SIM))./mean(Y_SIM); 
YSPdev=(YSP_SIM-mean(YSP_SIM))./mean(YSP_SIM);
yTdev=(ySIM-mean(ySIM))./mean(ySIM);

clear SS_Index SScontrol
SS_Index=find(SS==1);
SSSP_Index=find(SS_SP==1);
SS_Index=SS_Index(1+nbd:end-nbd); %Take out some events that might cause before/after periods to go out of bounds
SSSP_Index=SSSP_Index(1+nbd:end-nbd);

SScontrol=ismember(S_index,S_index(SS_Index));  % Track all the exogeneous states that occuered in SSs
SScontrol_Index=find(SScontrol==1);

SSomega=(SScontrol==1 & SS==0);
SSomega_Index=find(SSomega==1);

CdevSS=Cdev(SS_Index);
RERdevSS=RERdev(SS_Index);
CSPdevSS=CSPdev(SS_Index);
RERSPdevSS=RERSPdev(SS_Index);

fprintf('C drop in DE is: ');
fprintf('%5.4f \n ',  mean(CdevSS));
fprintf('C drop in SP is: ');
fprintf('%5.4f \n ',  mean(CSPdevSS));
fprintf('RER depreciation in DE is: ');
fprintf('%5.4f \n ',  mean(RERdevSS));
fprintf('RER depreciation in SP is: ');
fprintf('%5.4f \n ',  mean(RERSPdevSS));
fprintf('CA/Y | SS in DE is: ');
fprintf('%5.4f \n ',  mean(CA_SIM(SS_Index)));
fprintf('CA/Y | SS in SP is: ');
fprintf('%5.4f \n ',  mean(CASP_SIM(SS_Index)));

fprintf('Omega C in DE is: ');
fprintf('%5.4f \n ',  mean(Cdev(SS_Index))/mean(Cdev(SSomega_Index)));
fprintf('Omega C in SP is: ');
fprintf('%5.4f \n ',  mean(CSPdev(SS_Index))/mean(CSPdev(SSomega_Index)));
fprintf('Omega RER in DE is: ');
fprintf('%5.4f \n ',  mean(RERdev(SS_Index))/mean(RERdev(SSomega_Index)));
fprintf('Omega RER in SP is: ');
fprintf('%5.4f \n ',  mean(RERSPdev(SS_Index))/mean(RERSPdev(SSomega_Index)));
fprintf('Omega CA/Y | SS in DE is: ');
fprintf('%5.4f \n ',  mean(CA_SIM(SS_Index))-mean(CA_SIM(SSomega_Index)));
fprintf('Omega CA/Y | SS in SP is: ');
fprintf('%5.4f \n ',  mean(CASP_SIM(SS_Index))-mean(CASP_SIM(SSomega_Index)));   

nE=length(SS_Index);

for i=1:nE
    State_Event(i,:)=S_index(SS_Index(i)-nbd:SS_Index(i)+nbd);
    R_EventT(i,:)=RSIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    y_EventT(i,:)=ySIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    
    p_EventT(i,:)=(pSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(pSIM))./mean(pSIM);    
    CA_EventT(i,:)=CA_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);    
    bp_EventT(i,:)=(bpSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(bpSIM))./mean(bpSIM); 
    c_EventT(i,:)=(cSIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(cSIM))./mean(cSIM);
    RER_EventT(i,:)=RER_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);  
    
    Tao_EventT(i,:)=Tao_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);

    pSP_EventT(i,:)=(pSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(pSP_SIM))./mean(pSP_SIM); 
    CASP_EventT(i,:)=CASP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);
    bpSP_EventT(i,:)=(bpSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(bpSP_SIM))./mean(bpSP_SIM);
    cSP_EventT(i,:)=(cSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd)-mean(cSP_SIM))./mean(cSP_SIM);
    RERSP_EventT(i,:)=RERSP_SIM(SS_Index(i)-nbd:SS_Index(i)+nbd);   
end

Rh_Event=zeros(size(State_Event));
Rh_Event(State_Event<=NSS)=1;
Rl_Event=1-Rh_Event;
Rh_Event=mean(Rh_Event);
Rl_Event=mean(Rl_Event);

R_Event=mean(R_EventT);
y_Event=mean(y_EventT);
p_Event=mean(p_EventT);
CA_Event=mean(CA_EventT);
bp_Event=mean(bp_EventT);
c_Event=mean(c_EventT);
Tao_Event=mean(Tao_EventT);

pSP_Event=mean(pSP_EventT);
CASP_Event=mean(CASP_EventT);
bpSP_Event=mean(bpSP_EventT);
cSP_Event=mean(cSP_EventT);
RERSP_Event=mean(RERSP_EventT);
RER_Event=mean(RER_EventT);

fprintf('Avg Tax b4 SS is: ');
fprintf('%5.4f \n ',  Tao_Event(nbd));
fprintf('====================== \n ');

Output(12:13,:)=[Tao_Event(nbd)-1,0;0,0];

yt_Event=zeros(size(State_Event));
ytB=find(yt(1,:)==Z(1));
ytA=find(yt(1,:)==Z(2));
ytG=find(yt(1,:)==Z(3));

news_Event=zeros(size(State_Event));
newsB=[1,2,3];
newsB=[newsB,newsB+NSS];
newsA=[4:6];
newsA=[newsA,newsA+NSS];
newsG=[7:9];
newsG=[newsG,newsG+NSS];

for i=1:length(newsB)
    news_Event(State_Event==newsB(i))=1;
    news_Event(State_Event==newsA(i))=2;
    news_Event(State_Event==newsG(i))=3;
    yt_Event(State_Event==ytB(i))=1;
    yt_Event(State_Event==ytA(i))=2;
    yt_Event(State_Event==ytG(i))=3;
end
news_EventG=sum(news_Event==3)/length(news_Event);
news_EventA=sum(news_Event==2)/length(news_Event);
news_EventB=sum(news_Event==1)/length(news_Event);

yt_EventG=sum(yt_Event==3)/length(yt_Event);
yt_EventA=sum(yt_Event==2)/length(yt_Event);
yt_EventB=sum(yt_Event==1)/length(yt_Event);

Event=[-nbd:1:nbd];

%% 3. Crises breakdown
% Interest Regime Switch
clearvars -regexp ^CRS
Rb4=RSIM(SS_Index-1);
Rcris=RSIM(SS_Index);

varlist={'HH','HL','LH','LL'};

k=1;
for i=1:length(r)
    for j=1:length(r)
    CRS_SS.(varlist{k})=SS_Index(Rb4==r(i) & Rcris==r(j));
    k=k+1;
    end
end

CRS_SS.H=[CRS_SS.HH;CRS_SS.HL];

fprintf('C drop in DE is: ');
fprintf('%5.4f \n ',  mean(Cdev(CRS_SS.LH)));
fprintf('C drop in SP is: ');
fprintf('%5.4f \n ',  mean(CSPdev(CRS_SS.LH)));
fprintf('RER depreciation in DE is: ');
fprintf('%5.4f \n ',  mean(RERdev(CRS_SS.LH)));
fprintf('RER depreciation in SP is: ');
fprintf('%5.4f \n ',  mean(RERSPdev(CRS_SS.LH)));
fprintf('CA/Y | SS in DE is: ');
fprintf('%5.4f \n ',  mean(CA_SIM(CRS_SS.LH)));
fprintf('CA/Y | SS in SP is: ');
fprintf('%5.4f \n ',  mean(CASP_SIM(CRS_SS.LH)));
fprintf('Tax before | SSHL is: ');
fprintf('%5.4f \n ',  mean(Tao_SIM(CRS_SS.LH-1)));

% longer history
Rb42=RSIM(SS_Index-2);
CRS_SS.LLH=SS_Index(Rb4==r(2) & Rb42==r(2)& Rcris==r(1)); % High liquidity event is defined as two low R episodes preceeding a high R crisis

RswitchComp(1,:)=[mean(Cdev(CRS_SS.H)),mean(Cdev(CRS_SS.LLH))];
RswitchComp(2,:)=[-mean(RERdev(CRS_SS.H)),-mean(RERdev(CRS_SS.LLH))];
RswitchComp(3,:)=[mean(CA_SIM(CRS_SS.H)),mean(CA_SIM(CRS_SS.LLH))];
RswitchComp(4,:)=[mean(Tao_SIM(CRS_SS.H-1))-1,mean(Tao_SIM(CRS_SS.LLH-1))-1];
RswitchComp=RswitchComp*100;

% news

newsGb4=SS_Index(news_Event(:,nbd)==3);
newsBb4=SS_Index(news_Event(:,nbd)==1);
newsAb4=SS_Index(news_Event(:,nbd)==2);

newsComp(1,:)=[mean(Cdev(newsBb4)),mean(Cdev(newsAb4)),mean(Cdev(newsGb4))];
newsComp(2,:)=[-mean(RERdev(newsBb4)),-mean(RERdev(newsAb4)),-mean(RERdev(newsGb4))];
newsComp(3,:)=[mean(CA_SIM(newsBb4)),mean(CA_SIM(newsAb4)),mean(CA_SIM(newsGb4))];
newsComp(4,:)=[mean(Tao_SIM(newsBb4-1))-1,mean(Tao_SIM(newsAb4-1))-1,mean(Tao_SIM(newsGb4-1))-1];
newsComp=newsComp*100;